@@ -1,8 +1,34 @@
+"""
+Distribution Subclass Module
+
+Defines commonly used distributions as subclasses of the
+    RV class
+
+Continuous Distributions:
+    ArcSinRV(),ArcTanRV(alpha,phi),BetaRV(alpha,beta)
+    CauchyRV(a,alpha),ChiRV(N),ChiSquareRV(N),ErlangRV(theta,N)
+    ErrorRV(mu,alpha,d),ErrorIIRV(a,b,c),ExponentialRV(theta)
+    ExponentialPowerRV(theta,kappa),ExtremeValueRV(alpha,beta)
+    FRV(n1,n2),GammaRV(theta,kappa),GompertzRV(theta,kappa)
+    GeneralizedParetoRV(theta,delta,kappa),IDBRV(theta,delta,kappa)
+    InverseGaussianRV(theta,mu),InverseGammaRV(alpha,beta)
+    KSRV(n),LaPlaceRV(omega,theta), LogGammaRV(alpha,beta)
+    LogisticRV(kappa,theta),LogLogisticRV(theta,kappa)
+    LogNormalRV(mu,sigma),LomaxRV(kappa,theta)
+    MakehamRV(theta,delta,kappa),MuthRV(kappa),NormalRV(mu,sigma)
+    ParetoRV(theta,kappa),RayleighRV(theta),TriangularRV(a,b,c)
+    TRV(N),UniformRV(a,b),WeibullRV(theta,kappa)
+
+
+Discrete Distributions
+    BenfordRV(),BinomialRV(n,p),GeometricRV(p),PoissonRV(theta)
+"""
+
 from __future__ import division
 from sympy import (Symbol, symbols, oo, integrate, summation, diff,
                    exp, pi, sqrt, factorial, ln, floor, simplify,
-                   solve, nan, plot, Add, Mul, Integer, function,
-                   binomial)
+                   solve, nan, Add, Mul, Integer, function,
+                   binomial,gamma,cos)
 from random import random
 from .rv import (RV, RVError, CDF, CHF, HF, IDF, IDF, PDF, SF,
                  BootstrapRV, Convert)
@@ -27,16 +53,6 @@ x,y,z,t,v=symbols('x y z t v')
     along with this program.  If not, see <http://www.gnu.org/licenses/>
 """
 
-"""
-Distribution Subclass Module
-
-Defines commonly used distributions as subclasses of the
-    RV class
-
-"""
-
-
-
 def param_check(param):
     flag=True
     count=0
@@ -55,13 +71,26 @@ Continuous Distributions
 """
 
 class ArcSinRV(RV):
+    """
+    Procedure Name: ArcSinRV
+    Purpose: Creates an instance of the arc sin distribution
+    Arguments:  1. None
+    Output:     1. An arc sin random variable
+    """
     def __init__(self):
         X_dummy=RV(1/(pi*sqrt(x*(1-x))),[0,1])
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class ArcTanRV(RV):
+    """
+    Procedure Name: ArcTanRV
+    Purpose: Creates an instance of the arc tan distribution
+    Arguments:  1. alpha: a strictly positive parameter
+    Output:     1. An arc tan random variable
+    """
     def __init__(self,alpha=Symbol('alpha',positive=True),
                  phi=Symbol('phi')):
         # Return an error if invalid parameters are entered
@@ -77,8 +106,16 @@ class ArcTanRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class BetaRV(RV):
+    """
+    Procedure Name: BetaRV
+    Purpose: Creates an instance of the beta distribution
+    Arguments:  1. alpha: a strictly positive parameter
+                2. beta: a strictly positive parameter
+    Output:     1. A beta random variable
+    """
     def __init__(self,alpha=Symbol('alpha',positive=True),
                  beta=Symbol('beta'),positive=True):
         if alpha in [-oo,oo]:
@@ -94,8 +131,16 @@ class BetaRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class CauchyRV(RV):
+    """
+    Procedure Name: CauchyRV
+    Purpose: Creates an instance of the Cauchy distribution
+    Arguments:  1. a: a real valued parameter
+                2. alpha: a stictly positive parameter
+    Output:     1. A Cauchy random variable
+    """
     def __init__(self,a=Symbol('a'),
                  alpha=Symbol('alpha'),positive=True):
         if a in [-oo,oo]:
@@ -111,17 +156,31 @@ class CauchyRV(RV):
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
         self.parameter=[a,alpha]
+        self.cache=None
 
-    def variate(self,n=1,s='sim'):
+    def variate(self,n=1,s=None,method='special'):        
         # If no parameter is specified, return an error
         if param_check(self.parameter)==False:
             raise RVError('Not all parameters specified')
 
-        # Generate exponential variates
+        # Check to see if the user specified a valid method
+        method_list=['special','inverse']
+        if method not in method_list:
+            error_string='an invalid method was specified'
+            raise RVError(error_string)
+
+        # If the inverse method is specified, compute variates using
+        #   the IDF function
+        if method=='inverse':
+            Xidf=IDF(self)
+            varlist=[IDF(Xidf,random()) for i in range(1,n+1)]
+            return varlist
+
+        # Generate cauchy variates
         idf_func=self.parameter[0]-cot(pi*t)*self.parameter[1]
         varlist=[]
         for i in range(n):
-            if s=='sim':
+            if s==None:
                 val=random()
             else:
                 val=s
@@ -131,6 +190,12 @@ class CauchyRV(RV):
         return varlist
 
 class ChiRV(RV):
+    """
+    Procedure Name: ChiRV
+    Purpose: Creates an instance of the chi distribution
+    Arguments:  1. N: a positive integer parameter
+    Output:     1. A chi random variable
+    """
     def __init__(self,N=Symbol('N',positive=True,
                                integer=True)):
         if N.__class__.__name__!='Symbol':
@@ -142,8 +207,15 @@ class ChiRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class ChiSquareRV(RV):
+    """
+    Procedure Name: ChiSquareRV
+    Purpose: Creates an instance of the chi square distribution
+    Arguments:  1. N: a positive integer parameter
+    Output:     1. A chi squared random variable
+    """
     def __init__(self,N=Symbol('N',positive=True,
                                integer=True)):
         if N.__class__.__name__!='Symbol':
@@ -155,8 +227,16 @@ class ChiSquareRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class ErlangRV(RV):
+    """
+    Procedure Name: ErlangRV
+    Purpose: Creates an instance of the Erlang distribution
+    Arguments:  1. theta: a strictly positive parameter
+                2. N: a positive integer parameter
+    Output:     1. An erlang random variable
+    """
     def __init__(self,theta=Symbol('theta',positive=True),
                  N=Symbol('N',positive=True,integer=True)):
         if N.__class__.__name__!='Symbol':
@@ -175,8 +255,17 @@ class ErlangRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class ErrorRV(RV):
+    """
+    Procedure Name: ErrorRV
+    Purpose: Creates an instance of the error distribution
+    Arguments:  1. mu: a strictly positive parameter
+                2. alpha: a real valued parameter
+                3. d: a real valued parameter
+    Output:     1. An error random variable
+    """
     def __init__(self,mu=Symbol('mu',positive=True),
                  alpha=Symbol('alpha'),d=Symbol('d')):
         if mu.__class__.__name__!='Symbol':
@@ -193,8 +282,17 @@ class ErrorRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class ErrorIIRV(RV):
+    """
+    Procedure Name: ErrorIIRV
+    Purpose: Creates an instance of the error II distribution
+    Arguments:  1. a: a real valued parameter
+                2. b: a real valued parameter
+                3. c: a real valued parameter
+    Output:     1. An error II random variable
+    """
     def __init__(self,a=Symbol('a'),b=Symbol('b'),
                  c=Symbol('c')):
         if a in [-oo,oo]:
@@ -208,8 +306,15 @@ class ErrorIIRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class ExponentialRV(RV):
+    """
+    Procedure Name: ExponentialRV
+    Purpose: Creates an instance of the exponential distribution
+    Arguments:  1. theta: a strictly positive parameter
+    Output:     1. An exponential random variable
+    """
     def __init__(self,theta=Symbol('theta',positive=True)):
         if theta.__class__.__name__!='Symbol':
             if theta<=0:
@@ -223,17 +328,31 @@ class ExponentialRV(RV):
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
         self.parameter=[theta]
+        self.cache=None
 
-    def variate(self,n=1,s='sim'):
+    def variate(self,n=1,s=None,method='special'):       
         # If no parameter is specified, return an error
         if param_check(self.parameter)==False:
             raise RVError('Not all parameters specified')
 
+        # Check to see if the user specified a valid method
+        method_list=['special','inverse']
+        if method not in method_list:
+            error_string='an invalid method was specified'
+            raise RVError(error_string)
+
+        # If the inverse method is specified, compute variates using
+        #   the IDF function
+        if method=='inverse':
+            Xidf=IDF(self)
+            varlist=[IDF(Xidf,random()) for i in range(1,n+1)]
+            return varlist
+
         # Generate exponential variates
         idf_func=(-ln(1-t))/(self.parameter[0])
         varlist=[]
         for i in range(n):
-            if s=='sim':
+            if s==None:
                 val=random()
             else:
                 val=s
@@ -243,6 +362,13 @@ class ExponentialRV(RV):
         return varlist
 
 class ExponentialPowerRV(RV):
+    """
+    Procedure Name: ExponentialPowerRV
+    Purpose: Creates an instance of the exponential power distribution
+    Arguments:  1. theta: a strictly positive parameter
+                2. kappa: a strictly positive parameter
+    Output:     1. An exponential power random variable
+    """
     def __init__(self,theta=Symbol('theta',positive=True),
                  kappa=Symbol('kappa',positive=True)):
         if theta.__class__.__name__!='Symbol':
@@ -256,18 +382,32 @@ class ExponentialPowerRV(RV):
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
         self.parameter=[theta,kappa]
+        self.cache=None
 
-    def variate(self,n=1,s='sim'):
+    def variate(self,n=1,s=None,method='special'):
         # If no parameter is specified, return an error
         if param_check(self.parameter)==False:
             raise RVError('Not all parameters specified')
 
+        # Check to see if the user specified a valid method
+        method_list=['special','inverse']
+        if method not in method_list:
+            error_string='an invalid method was specified'
+            raise RVError(error_string)
+
+        # If the inverse method is specified, compute variates using
+        #   the IDF function
+        if method=='inverse':
+            Xidf=IDF(self)
+            varlist=[IDF(Xidf,random()) for i in range(1,n+1)]
+            return varlist
+
         # Generate exponential power variates
         idf_func=exp((-ln(self.parameter[0])+ln(ln(1-ln(1-s))))/
                      self.parameter[1])
         varlist=[]
         for i in range(n):
-            if s=='sim':
+            if s==None:
                 val=random()
             else:
                 val=s
@@ -277,6 +417,13 @@ class ExponentialPowerRV(RV):
         return varlist
 
 class ExtremeValueRV(RV):
+    """
+    Procedure Name: ExtremeValueRV
+    Purpose: Creates an instance of the extreme value distribution
+    Arguments:  1. alpha: a real valued parameter
+                2. beta: a real valued parameter
+    Output:     1. An extreme value random variable
+    """
     def __init__(self,alpha=Symbol('alpha'),beta=Symbol('beta')):
         if alpha in [-oo,oo]:
             if beta in [-oo,oo]:
@@ -288,15 +435,30 @@ class ExtremeValueRV(RV):
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
         self.parameter=[alpha,beta]
+        self.cache=None
 
-    def variate(self,n=1,s='sim'):
+    def variate(self,n=1,s=None,method='special'):
         if param_check(self.parameter)==False:
             raise RVError('Not all parameters specified')
 
+        # Check to see if the user specified a valid method
+        method_list=['special','inverse']
+        if method not in method_list:
+            error_string='an invalid method was specified'
+            raise RVError(error_string)
+
+        # If the inverse method is specified, compute variates using
+        #   the IDF function
+        
+        if method=='inverse':
+            Xidf=IDF(self)
+            varlist=[IDF(Xidf,random()) for i in range(1,n+1)]
+            return varlist
+
         idf_func=(ln(self.parameter[0])+ln(ln(-1/(t-1))))/self.parameter[1]
         varlist=[]
         for i in range(n):
-            if s=='sim':
+            if s==None:
                 val=random()
             else:
                 val=s
@@ -305,6 +467,13 @@ class ExtremeValueRV(RV):
         return varlist
 
 class FRV(RV):
+    """
+    Procedure Name: FRV
+    Purpose: Creates an instance of the f distribution
+    Arguments:  1. n1: a strictly positive parameter
+                2. n2: a strictly positive parameter
+    Output:     1. A chi squared random variable
+    """
     def __init__(self,n1=Symbol('n1',positive=True),
                  n2=Symbol('n2',positive=True)):
         if n1.__class__.__name__!='Symbol':
@@ -321,8 +490,16 @@ class FRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class GammaRV(RV):
+    """
+    Procedure Name: GammaRV
+    Purpose: Creates an instance of the gamma distribution
+    Arguments:  1. theta: a strictly positive parameter
+                2. kappa: a strictly positive parameter
+    Output:     1. A chi squared random variable
+    """
     def __init__(self,theta=Symbol('theta',positive=True),
                  kappa=Symbol('kappa',positive=True)):
         if theta.__class__.__name__!='Symbol':
@@ -339,8 +516,17 @@ class GammaRV(RV):
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
         self.parameter=[theta,kappa]
+        self.cache=None
 
 class GeneralizedParetoRV(RV):
+    """
+    Procedure Name: GeneralizedParetoRV
+    Purpose: Creates an instance of the generalized pareto distribution
+    Arguments:  1. theta: a strictly positive parameter
+                2. delta: a real valued parameter
+                3. kappa: a real valued parameter
+    Output:     1. A generalized pareto random variable
+    """
     def __init__(self,theta=Symbol('theta',positive=True),
                  delta=Symbol('delta'),kappa=Symbol('kappa')):
         if theta.__class__.__name__!='Symbol':
@@ -355,8 +541,16 @@ class GeneralizedParetoRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class GompertzRV(RV):
+    """
+    Procedure Name: GompertzRV
+    Purpose: Creates an instance of the gompertz distribution
+    Arguments:  1. theta: a strictly positive parameter
+                2. kappa: a real valued parameter
+    Output:     1. A gompertz random variable
+    """
     def __init__(self,theta=Symbol('theta',positive=True),
                  kappa=Symbol('kappa')):
         if theta.__class__.__name__!='Symbol':
@@ -372,17 +566,31 @@ class GompertzRV(RV):
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
         self.parameter=[theta,kappa]
+        self.cache=None
 
-    def variate(self,n=1,s='sim'):
+    def variate(self,n=1,s=None,method='special'):
         if param_check(self.parameter)==False:
             raise RVError('Not all parameters specified')
 
+        # Check to see if the user specified a valid method
+        method_list=['special','inverse']
+        if method not in method_list:
+            error_string='an invalid method was specified'
+            raise RVError(error_string)
+
+        # If the inverse method is specified, compute variates using
+        #   the IDF function
+        if method=='inverse':
+            Xidf=IDF(self)
+            varlist=[IDF(Xidf,random()) for i in range(1,n+1)]
+            return varlist
+
         idf_func=-((ln(self.parameter[0])-
                    ln(self.parameter[0]-ln(1-t)*ln(self.parameter[1])))
                    /ln(self.parameter[1]))
         varlist=[]
         for i in range(n):
-            if s=='sim':
+            if s==None:
                 val=random()
             else:
                 val=s
@@ -391,6 +599,14 @@ class GompertzRV(RV):
         return varlist
 
 class IDBRV(RV):
+    """
+    Procedure Name: IDBRV
+    Purpose: Creates an instance of the idb distribution
+    Arguments:  1. theta: a real valued parameter
+                2. delta: a real valued parameter
+                3. kappa: a real valued parameter
+    Output:     1. An idb random variable
+    """
     def __init__(self,theta=Symbol('theta'),delta=Symbol('delta'),
                  kappa=Symbol('kappa')):
         if theta in [-oo,oo] or delta in [-oo,oo] or kappa in [-oo,oo]:
@@ -401,8 +617,16 @@ class IDBRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class InverseGaussianRV(RV):
+    """
+    Procedure Name: InverseGaussianRV
+    Purpose: Creates an instance of the inverse gaussian distribution
+    Arguments:  1. theta: a strictly positive parameter
+                2. mu: a strictly positive parameter
+    Output:     1. An inverse gaussian random variable
+    """
     def __init__(self,theta=Symbol('theta',positive=True),
                  mu=Symbol('mu',positive=True)):
         if theta.__class__.__name__!='Symbol':
@@ -419,9 +643,17 @@ class InverseGaussianRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 
 class InverseGammaRV(RV):
+    """
+    Procedure Name: InverseGammaRV
+    Purpose: Creates an instance of the inverse gamma distribution
+    Arguments:  1. alpha: a strictly positive parameter
+                2. beta: a strictly positive parameter
+    Output:     1. An inverse gamma random variable
+    """
     def __init__(self,alpha=Symbol('alpha',positive=True),
                  beta=Symbol('beta',positive=True)):
         if alpha.__class__.__name__!='Symbol':
@@ -437,8 +669,15 @@ class InverseGammaRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class KSRV(RV):
+    """
+    Procedure Name: KSRV
+    Purpose: Creates an instance of the kolmogoroff-smirnov distribution
+    Arguments:  1. n: a positive integer parameter
+    Output:     1. A kolmogoroff-smirnov random variable
+    """
     def __init__(self,n=Symbol('n',positive=True,
                                integer=True)):
         if n.__class__.__name__!='Symbol':
@@ -593,8 +832,16 @@ class KSRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class LaPlaceRV(RV):
+    """
+    Procedure Name: LaPlaceRV
+    Purpose: Creates an instance of the LaPlace distribution
+    Arguments:  1. omega: a strictly positive parameter
+                2. theta: a real valued parameter
+    Output:     1. A LaPlace random variable
+    """
     def __init__(self,omega=Symbol('omega',positive=True),
                  theta=Symbol('theta')):
         if omega.__class__.__name__!='Symbol':
@@ -607,8 +854,16 @@ class LaPlaceRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
         
 class LogGammaRV(RV):
+    """
+    Procedure Name: LogGammaRV
+    Purpose: Creates an instance of the log gamma distribution
+    Arguments:  1. alpha: a strictly positive parameter
+                2. beta: a strictly positive parameter
+    Output:     1. A log gamma random variable
+    """
     def __init__(self,alpha=Symbol('alpha',positive=True),
                  beta=Symbol('beta',positive=True)):
         if alpha.__class__.__name__!='Symbol':
@@ -624,8 +879,16 @@ class LogGammaRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class LogisticRV(RV):
+    """
+    Procedure Name: LogisticRV
+    Purpose: Creates an instance of the logistic distribution
+    Arguments:  1. kappa: a strictly positive parameter
+                2. theta: a strictly positive parameter
+    Output:     1. A logistic random variable
+    """
     def __init__(self,kappa=Symbol('kappa',positive=True),
                  theta=Symbol('theta',positive=True)):
         if kappa.__class__.__name__!='Symbol':
@@ -642,16 +905,30 @@ class LogisticRV(RV):
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
         self.parameter=[kappa,theta]
+        self.cache=None
 
-    def variate(self,n=1,s='sim'):
+    def variate(self,n=1,s=None,method='special'):
         if param_check(self.parameter)==False:
             raise RVError('Not all parameters specified')
 
+        # Check to see if the user specified a valid method
+        method_list=['special','inverse']
+        if method not in method_list:
+            error_string='an invalid method was specified'
+            raise RVError(error_string)
+
+        # If the inverse method is specified, compute variates using
+        #   the IDF function
+        if method=='inverse':
+            Xidf=IDF(self)
+            varlist=[IDF(Xidf,random()) for i in range(1,n+1)]
+            return varlist
+
         idf_func=-((ln(-t/(t-1))+self.parameter[0]*ln(self.parameter[1]))/
                    self.parameter[1])
         varlist=[]
         for i in range(n):
-            if s=='sim':
+            if s==None:
                 val=random()
             else:
                 val=s
@@ -660,6 +937,13 @@ class LogisticRV(RV):
         return varlist
 
 class LogLogisticRV(RV):
+    """
+    Procedure Name: LogLogisticRV
+    Purpose: Creates an instance of the log logistic distribution
+    Arguments:  1. theta: a strictly positive parameter
+                2. kappa: a strictly positive parameter
+    Output:     1. A chi squared random variable
+    """
     def __init__(self,theta=Symbol('theta',positive=True),
                  kappa=Symbol('kappa',positive=True)):
         if kappa.__class__.__name__:
@@ -676,16 +960,30 @@ class LogLogisticRV(RV):
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
         self.parameter=[theta,kappa]
+        self.cache=None
 
-    def variate(self,n=1,s='sim'):
+    def variate(self,n=1,s=None,method='special'):
         if param_check(self.parameter)==False:
             raise RVError('Not all parameters specified')
 
+        # Check to see if the user specified a valid method
+        method_list=['special','inverse']
+        if method not in method_list:
+            error_string='an invalid method was specified'
+            raise RVError(error_string)
+
+        # If the inverse method is specified, compute variates using
+        #   the IDF function
+        if method=='inverse':
+            Xidf=IDF(self)
+            varlist=[IDF(Xidf,random()) for i in range(1,n+1)]
+            return varlist
+
         idf_func=exp((ln(-t/(t-1))-self.parameter[1]*ln(self.parameter[0]))/
                      self.parameter[1])
         varlist=[]
         for i in range(n):
-            if s=='sim':
+            if s==None:
                 val=random()
             else:
                 val=s
@@ -694,6 +992,13 @@ class LogLogisticRV(RV):
         return varlist
     
 class LogNormalRV(RV):
+    """
+    Procedure Name: LogNormalRV
+    Purpose: Creates an instance of the log normal distribution
+    Arguments:  1. mu: a real valued parameter
+                2. sigma: a strictly positive parameter
+    Output:     1. A log normal random variable
+    """
     def __init__(self,mu=Symbol('mu'),
                  sigma=Symbol('sigma',positive=True)):
         if sigma.__class__.__name__!='Symbol':
@@ -708,8 +1013,16 @@ class LogNormalRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class LomaxRV(RV):
+    """
+    Procedure Name: LomaxRV
+    Purpose: Creates an instance of the lomax distribution
+    Arguments:  1. kappa: a strictly positive parameter
+                2. theta: a strictly positive parameter
+    Output:     1. A lomax random variable
+    """
     def __init__(self,kappa=Symbol('kappa',positive=True),
                  theta=Symbol('theta',positive=True)):
         if kappa.__class__.__name__!='Symbol':
@@ -725,15 +1038,29 @@ class LomaxRV(RV):
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
         self.parameters=[kappa,theta]
+        self.cache=None
 
-    def variate(self,n=1,s='sim'):
+    def variate(self,n=1,s=None,method='special'):
         if param_check(self.parameter)==False:
             raise RVError('Not all parameters specified')
 
+        # Check to see if the user specified a valid method
+        method_list=['special','inverse']
+        if method not in method_list:
+            error_string='an invalid method was specified'
+            raise RVError(error_string)
+
+        # If the inverse method is specified, compute variates using
+        #   the IDF function
+        if method=='inverse':
+            Xidf=IDF(self)
+            varlist=[IDF(Xidf,random()) for i in range(1,n+1)]
+            return varlist
+
         idf_func=((1-t)**(1/self.parameter[0])-1)/self.parameter[1]
         varlist=[]
         for i in range(n):
-            if s=='sim':
+            if s==None:
                 val=random()
             else:
                 val=s
@@ -742,6 +1069,14 @@ class LomaxRV(RV):
         return varlist
 
 class MakehamRV(RV):
+    """
+    Procedure Name: MakehamRV
+    Purpose: Creates an instance of the Makeham distribution
+    Arguments:  1. theta: a strictly positive parameter
+                2. delta: a strictly positive parameter
+                3: kappa: a strictly positive parameter
+    Output:     1. A log normal random variable
+    """
     def __init__(self,theta=Symbol('theta',positive=True),
                  delta=Symbol('delta',positive=True),
                  kappa=Symbol('kappa')):
@@ -758,8 +1093,15 @@ class MakehamRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class MuthRV(RV):
+    """
+    Procedure Name: MuthRV
+    Purpose: Creates an instance of the Muth distribution
+    Arguments:  1. kappa: a strictly positive parameter
+    Output:     1. A log normal random variable
+    """
     def __init__(self,kappa=Symbol('kappa',positive=True)):
         if kappa.__class__.__name__!='Symbol':
             if kappa<=0:
@@ -774,8 +1116,16 @@ class MuthRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class NormalRV(RV):
+    """
+    Procedure Name: NormalRV
+    Purpose: Creates an instance of the normal distribution
+    Arguments:  1. mu: a real valued parameter
+                2. sigma: a strictly positive parameter
+    Output:     1. A normal random variable
+    """
     def __init__(self,mu=Symbol('mu'),
                  sigma=Symbol('sigma',positive=True)):
         if sigma.__class__.__name__!='Symbol':
@@ -791,8 +1141,46 @@ class NormalRV(RV):
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
         self.parameter=[mu,sigma]
+        self.cache=None
+
+    def variate(self,n=1,s=None,method='special'):
+        # If no parameter is specified, return an error
+        if param_check(self.parameter)==False:
+            raise RVError('Not all parameters specified')
+
+        # Check to see if the user specified a valid method
+        method_list=['special','inverse']
+        if method not in method_list:
+            error_string='an invalid method was specified'
+            raise RVError(error_string)
+
+        # If the inverse method is specified, compute variates using
+        #   the IDF function
+        if method=='inverse':
+            Xidf=IDF(self)
+            varlist=[IDF(Xidf,random()) for i in range(1,n+1)]
+            return varlist
+
+        # Otherwise, use the Box-Muller method to compute variates
+        mean=self.parameter[0];var=self.parameter[1]
+        U=UniformRV(0,1)
+        Z1=lambda (val1,val2): sqrt(-2*ln(val1))*cos(2*pi*val2).evalf()
+        gen_uniform=lambda x: U.variate(n=1)[0]
+        val_pairs=[(gen_uniform(1),gen_uniform(1)) for i in range(1,n+1)]
+        varlist=[Z1(pair) for pair in val_pairs]
+        normlist=[(mean+sqrt(var)*val).evalf() for val in varlist]
+        return normlist
+        
+        
 
 class ParetoRV(RV):
+    """
+    Procedure Name: ParetoRV
+    Purpose: Creates an instance of the pareto distribution
+    Arguments:  1. theta: a strictly positive parameter
+                2. kappa: a strictly positive parameter
+    Output:     1. A Paerto random variable
+    """
     def __init__(self,theta=Symbol('theta',positive=True),
                  kappa=Symbol('kappa',positive=True)):
         if theta.__class__.__name__!='Symbol':
@@ -807,8 +1195,15 @@ class ParetoRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class RayleighRV(RV):
+    """
+    Procedure Name: RayleighRV
+    Purpose: Creates an instance of the Rayleigh distribution
+    Arguments:  1. theta: a strictly positive parameter
+    Output:     1. A log normal random variable
+    """
     def __init__(self,theta=Symbol('theta',positive=True)):
         if theta.__class__.__name__!='Symbol':
             if theta<=0:
@@ -820,8 +1215,18 @@ class RayleighRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class TriangularRV(RV):
+    """
+    Procedure Name: TriangularRV
+    Purpose: Creates an instance of the triangular distribution
+    Arguments:  1. a: a real valued parameter
+                2. b: a real valued parameter
+                3. c: a real valued parameter
+                ** Note: a<b<c ***
+    Output:     1. A triangular variable
+    """
     def __init__(self,a=Symbol('a'),b=Symbol('b'),c=Symbol('c')):
         if a.__class__.__name__!='Symbol':
             if b.__class__.__name__!='Symbol':
@@ -838,8 +1243,15 @@ class TriangularRV(RV):
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
         self.parameter=[a,b,c]
+        self.cache=None
 
 class TRV(RV):
+    """
+    Procedure Name: TRV
+    Purpose: Creates an instance of the t distribution
+    Arguments:  1. N: a positive integer parameter
+    Output:     1. A log normal random variable
+    """
     def __init__(self,N=Symbol('N'),positive=True,integer=True):
         if N.__class__.__name__!='Symbol':
             if N<=0:
@@ -851,8 +1263,17 @@ class TRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class UniformRV(RV):
+    """
+    Procedure Name: UniformRV
+    Purpose: Creates an instance of the uniform distribution
+    Arguments:  1. a: a real valued parameter
+                2. b: a real valued parameter
+                ** Note: b>a **
+    Output:     1. A uniform random variable
+    """
     def __init__(self,a=Symbol('a'),b=Symbol('b')):
         if a.__class__.__name__!='Symbol':
             if b.__class__.__name__!='Symbol':
@@ -867,17 +1288,31 @@ class UniformRV(RV):
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
         self.parameter=[a,b]
+        self.cache=None
 
-    def variate(self,n=1,s='sim'):
+    def variate(self,n=1,s=None,method='special'):
         # If no parameter is specified, return an error
         if param_check(self.parameter)==False:
             raise RVError('Not all parameters specified')
 
+        # Check to see if the user specified a valid method
+        method_list=['special','inverse']
+        if method not in method_list:
+            error_string='an invalid method was specified'
+            raise RVError(error_string)
+
+        # If the inverse method is specified, compute variates using
+        #   the IDF function
+        if method=='inverse':
+            Xidf=IDF(self)
+            varlist=[IDF(Xidf,random()) for i in range(1,n+1)]
+            return varlist
+
         # Generate uniform variates
         idf_func=-t*self.parameter[0]+t*self.parameter[1]+self.parameter[0]
         varlist=[]
         for i in range(n):
-            if s=='sim':
+            if s==None:
                 val=random()
             else:
                 val=s
@@ -886,7 +1321,14 @@ class UniformRV(RV):
         varlist.sort()
         return varlist
 
-class WeibullRV(RV):   
+class WeibullRV(RV):
+    """
+    Procedure Name: WeibullRV
+    Purpose: Creates an instance of the weibull distribution
+    Arguments:  1. theta: a strictly positive parameter
+                2. kappa: a strictly positive parameter
+    Output:     1. A weibull random variable
+    """
     def __init__(self,theta=Symbol('theta',positive=True),
                  kappa=Symbol('kappa',positive=True)):
         if theta.__class__.__name__!='Symbol':
@@ -903,19 +1345,33 @@ class WeibullRV(RV):
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
         self.parameter=[theta,kappa]
+        self.cache=None
 
 
-    def variate(self,n=1,s='sim'):
+    def variate(self,n=1,s=None,method='special'):
         # If no parameter is specified, return an error
         if param_check(self.parameter)==False:
             raise RVError('Not all parameters specified')
 
+        # Check to see if the user specified a valid method
+        method_list=['special','inverse']
+        if method not in method_list:
+            error_string='an invalid method was specified'
+            raise RVError(error_string)
+
+        # If the inverse method is specified, compute variates using
+        #   the IDF function
+        if method=='inverse':
+            Xidf=IDF(self)
+            varlist=[IDF(Xidf,random()) for i in range(1,n+1)]
+            return varlist
+
         # Generate weibull variates
         idf_func=exp(-(-ln(-ln(1-t))+self.parameter[1]*ln(self.parameter[0]))/
                      self.parameter[1])
         varlist=[]
         for i in range(n):
-            if s=='sim':
+            if s==None:
                 val=random()
             else:
                 val=s
@@ -930,13 +1386,27 @@ Discrete Distributions
 """
 
 class BenfordRV(RV):
+    """
+    Procedure Name: BenfordRV
+    Purpose: Creates an instance of the Benford distribution
+    Arguments:  1. None
+    Output:     1. A Benford random variable
+    """
     def __init__(self):
         X_dummy=RV([(ln((1/x)+1))/(ln(10))],[1,9],['Discrete','pdf'])
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class BinomialRV(RV):
+    """
+    Procedure Name: BinomialRV
+    Purpose: Creates an instance of the binomial distribution
+    Arguments:  1. N: a positive integer parameter
+                2. p: a positive parameter between 0 and 1
+    Output:     1. A binomial random variable
+    """
     def __init__(self,N=Symbol('N',positive=True,integer=True),
                  p=Symbol('p',positive=True)):
         if N.__class__.__name__!='Symbol':
@@ -954,8 +1424,15 @@ class BinomialRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class GeometricRV(RV):
+    """
+    Procedure Name: GeometricRV
+    Purpose: Creates an instance of the geometric distribution
+    Arguments:  1. p: a positive parameter between 0 and 1
+    Output:     1. A geometric random variable
+    """
     def __init__(self,p=Symbol('p',positive=True)):
         if p.__class__.__name__!='Symbol':
             if p<=0 or p>=1:
@@ -965,8 +1442,15 @@ class GeometricRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
 class PoissonRV(RV):
+    """
+    Procedure Name: PoissonRV
+    Purpose: Creates an instance of the poisson distribution
+    Arguments:  1. theta: a strictly positive parameter
+    Output:     1. A poisson random variable
+    """
     def __init__(self,theta=Symbol('theta',positive=True)):
         if theta.__class__.__name__!='Symbol':
             if theta<=0:
@@ -980,4 +1464,5 @@ class PoissonRV(RV):
         self.func=X_dummy.func
         self.support=X_dummy.support
         self.ftype=X_dummy.ftype
+        self.cache=None
 
